@@ -9,7 +9,7 @@ use strict;
 use vars qw($VERSION);
 use Text::Template::Simple::Caller;
 
-$VERSION = '0.10';
+$VERSION = '0.54_01';
 
 sub stack { # just a wrapper
    my $opt = shift || {};
@@ -24,7 +24,7 @@ __END__
 
 =head1 NAME
 
-Text::Template::Simple::Dummy - Container class for Text::Template::Simple
+Text::Template::Simple::Dummy - Container class
 
 =head1 SYNOPSIS
 
@@ -58,7 +58,7 @@ Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004-2007 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
 
 =head1 LICENSE
 
