@@ -1,20 +1,17 @@
 package Text::Template::Simple::IO;
 use strict;
 use vars qw($VERSION);
-use constant IO_PARENT => 0;
 use Text::Template::Simple::Constants;
 use Text::Template::Simple::Util qw( DEBUG LOG ishref binary_mode );
-use Text::Template::Simple::Cache::ID;
 use Carp qw( croak );
 
-$VERSION = '0.51';
+$VERSION = '0.54_01';
 
 sub new {
-   my $class  = shift;
-   my $parent = shift || croak "Parent object is missing";
-   my $self   = [undef];
-   bless $self, $class;
-   $self->[IO_PARENT] = $parent;
+   my $class = shift;
+   my $layer = shift;
+   my $self  = bless do { \my $anon }, $class;
+   $$self    = $layer if defined $layer;
    $self;
 }
 
@@ -50,10 +47,9 @@ sub validate {
 sub layer {
    return if ! NEW_PERL;
    my $self   = shift;
-   my $parent = $self->[IO_PARENT];
    my $fh     = shift || croak "layer(): Filehandle is absent";
-   my $layer  = $parent->[IOLAYER]; # || croak "_iolayer(): I/O Layer is absent";
-   binary_mode( $fh, $layer ) if $parent->[IOLAYER];
+   my $layer  = $$self; # || croak "_iolayer(): I/O Layer is absent";
+   binary_mode( $fh, $layer ) if $layer;
    return;
 }
 
@@ -62,21 +58,31 @@ sub slurp {
    require Fcntl;
    my $self = shift;
    my $file = shift;
-   my $fh   = IO::File->new;
-   $fh->open($file, 'r') or croak "Error opening $file for reading: $!";
-   flock $fh, Fcntl::LOCK_SH() if IS_FLOCK;
+   my($fh, $seek);
+   LOG(IO_SLURP => $file) if DEBUG;
+
+   if ( fileno $file ) {
+      $fh   = $file;
+      $seek = 1;
+   }
+   else {
+      $fh = IO::File->new;
+      $fh->open($file, 'r') or croak "Error opening '$file' for reading: $!";
+   }
+
+   flock $fh,    Fcntl::LOCK_SH()  if IS_FLOCK;
+   seek  $fh, 0, Fcntl::SEEK_SET() if IS_FLOCK && $seek;
    $self->layer( $fh );
-   local $/;
-   my $tmp = <$fh>;
+   my $tmp = do { local $/; <$fh> };
    flock $fh, Fcntl::LOCK_UN() if IS_FLOCK;
-   $fh->close;
+   close $fh if ! $seek; # close only if we opened this
    return $tmp;
 }
 
 sub DESTROY {
    my $self = shift;
    LOG( DESTROY => ref $self ) if DEBUG;
-   @{$self} = ();
+   $$self = undef;
    return;
 }
 
@@ -98,9 +104,9 @@ TODO
 
 =head1 METHODS
 
-=head2 new PARENT_OBJECT
+=head2 new IO_LAYER
 
-Constructor. Accepts a C<Text::Template::Simple> object as the parameter.
+Constructor. Accepts an I/O layer name as the parameter.
 
 =head2 layer FH
 
